import 'package:example/presenter.dart';
import 'package:example/state/state.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:osam/osam.dart';

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  final store = Store(AppState());
  runApp(MyApp(
    store: store,
    key: GlobalKey(),
  ));
}

enum EventType { increment, incrementValue }

class MyApp extends StatelessWidget {
  final Store<AppState> store;

  const MyApp({Key key, this.store}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: StoreProvider(
        child: MyHomePage(),
        store: store,
      ),
    );
  }
}

class MyHomePage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Center(
        child: FutureBuilder(
          future: Future.delayed(Duration(seconds: 4)),
          builder: (context, snapshot) {

            return Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                Text(
                  'You have pushed the button this many times:',
                ),
                PresenterProvider(
                  key: ValueKey('counter'),
                  presenter: ExamplePresenter(),
                  child: Button(),
                )
              ],
            );
          }
        ),
      ),
    );
  }
}

class Button extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final presenter = PresenterProvider.of<ExamplePresenter>(context);
    return Column(
      mainAxisAlignment: MainAxisAlignment.end,
      children: <Widget>[
        StreamBuilder(
          stream: presenter.stream,
          builder: (ctx, AsyncSnapshot<String> snapshot) {
            print(snapshot.data);
            return Text(snapshot?.data ?? 'we');
          },
        ),
        RaisedButton(
          child: Text('inc'),
          onPressed: () {
            presenter.increment();
          },
        )
      ],
    );
  }
}
